/*
 * RemoteInterfaces.java
 *
 * Created on 02 February 2007, 11:58
 *
 *
 * www.starjar.com
 *             +
 *     +       *       +
 *       *     *     *
 *         *   *   *
 *           * * *
 *   - * * * * * * * * * -
 *           * * *
 *         *   *   *
 *       *     *     *
 *     +       *       +
 *             +
 *
 *     SSSS  TTTTTTT    A     RRRRR   JJJJJJJ    A     RRRRR
 *    S    S    T      A A    R    R     J      A A    R    R
 *    S         T     A   A   R    R     J     A   A   R    R
 *      SSS     T    A     A  RRRRR      J    A     A  RRRRR
 *         S    T   AAAAAAAAA R   R      J   AAAAAAAAA R   R
 *    S    S    T   A       A R    R  J  J   A       A R    R  ..
 *     SSS      T   A       A R    R   JJJ   A       A R    R  ..
 *
 * Copyright (c) 2007 Peter Henderson. All Rights Reserved.
 */

package breakglassfishpermissions;

import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Loging and Remote Interface cache
 *
 * As looking up remote interfaces is SLOW cache them, and only load them on 
 * demand.
 * 
 *
 * @author peter
 */
public class RemoteInterfaces {

    static Logger log = Logger.getLogger(RemoteInterfaces.class.getName());
    
    private static Context initial = null;

    
    /**
     * Create the initial context and then login.
     */
    public static boolean createInitialContextAndLogin(String uid, String pwd) {
        boolean loginOK = false;

        try {
            login(uid, pwd);

            initial = new InitialContext();

            // IF we get this far log in was successful.
            loginOK = true;
        } catch(Exception e) {
            log.log(Level.SEVERE, "Problem creating initial context", e);
            initial = null;
        }

        return loginOK;
    }
    
    
    @SuppressWarnings("unchecked")
    public static boolean login(String uid, String pwd) {
        boolean loginOk = false;

        try {
            Class clz = Class.forName("com.sun.appserv.security.ProgrammaticLogin");
            Method loginMethod = clz.getMethod("login", String.class, String.class);
            Method logoutMethod = clz.getMethod("logout");
            
            Object loginApi = clz.newInstance();
            
            System.out.println("About to invoke Login");
            Object result = loginMethod.invoke(loginApi, uid, pwd);
            System.out.println("Login called OK. login result = " + result);
            
            //Method[] meths = clz.getMethods();
            //for( int i=0; i<meths.length; i++) {
            //    System.out.println("Meth[" + i + "] = " + meths[i].getName() );
            //}
            
            loginOk = true;
            
        } catch(Exception e) {
            log.log(Level.SEVERE, "Problem logging in", e);
        }
        return loginOk;
    }
    
    
    
    
    private static SessionOneRemote sessionOneRemote = null;
    
    public static synchronized SessionOneRemote getSessionOneRemote() {
        if (sessionOneRemote==null) {
            try {
                log.severe("Loading SessionOneRemote " + Thread.currentThread().getName() );
                sessionOneRemote = (SessionOneRemote)initial.lookup(
                        "breakglassfishpermissions.SessionOneRemote");
            } catch(NamingException ne) {
                log.log(Level.SEVERE, "Problem looking up Remote Interface", ne);
            }
        }
        return sessionOneRemote;
    }

    public static synchronized void setSessionOneRemote(SessionOneRemote r) {
        sessionOneRemote = r;
    }
    
    private static SessionTwoRemote sessionTwoRemote = null;
    
    public static synchronized SessionTwoRemote getSessionTwoRemote() {
        if (sessionTwoRemote==null) {
            try {
                log.severe("Loading SessionTwoRemote " + Thread.currentThread().getName() );
                sessionTwoRemote = (SessionTwoRemote)initial.lookup(
                        "breakglassfishpermissions.SessionTwoRemote");
            } catch(NamingException ne) {
                log.log(Level.SEVERE, "Problem looking up Remote Interface", ne);
            }
        }
        return sessionTwoRemote;
    }

    public static synchronized void setSessionTwoRemote(SessionTwoRemote r) {
        sessionTwoRemote = r;
    }
    
    
    private static SessionThreeRemote sessionThreeRemote = null;
    
    public static synchronized SessionThreeRemote getSessionThreeRemote() {
        if (sessionThreeRemote==null) {
            try {
                log.severe("Loading SessionThreeRemote " + Thread.currentThread().getName() );
                sessionThreeRemote = (SessionThreeRemote)initial.lookup(
                        "breakglassfishpermissions.SessionThreeRemote");
            } catch(NamingException ne) {
                log.log(Level.SEVERE, "Problem looking up Remote Interface", ne);
            }
        }
        return sessionThreeRemote;
    }
    
    public static synchronized void setSessionThreeRemote(SessionThreeRemote r) {
        sessionThreeRemote = r;
    }
    
}
